# -*- coding: utf-8 -*-
import logging

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction


class Command(BaseCommand):
    help = "Load initial data"
    logger = logging.getLogger('file')

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '-c', '--console', action='store_true', default=False,
            help='Debug - write logging to console',
        )
        parser.add_argument(
            '-l', '--debug-level', default='info',
            help='Set debug level (debug, info, warnings) for console',
        )
        parser.add_argument(
            '-d', '--dev', action='store_true', default=False,
            help='Load also `dev` data',
        )

    def set_logging(self, options: dict) -> None:
        debug_level = getattr(logging, options.get('debug_level').upper())
        self.logger.setLevel(debug_level)
        self.logger.handlers[0].setLevel(debug_level)

        if options.get('console'):
            console_handler = logging.StreamHandler()
            console_handler.setLevel(debug_level)
            self.logger.addHandler(console_handler)
            self.logger.setLevel(debug_level)

    def handle(self, *args, **options) -> None:
        try:
            self.set_logging(options)
            self.load_data(dev=options['dev'])
            self.stdout.write(f'Execution completed')
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Execution failed: {e}'))
            self.logger.error(e)

    @transaction.atomic
    def load_data(self, dev: bool = False) -> None:
        for app in getattr(settings, 'THX_CORE_INIT_DATA_APPS', []):
            call_command(
                "loaddata",
                settings.THX_CORE_INIT_DATA_FILE_NAME,
                app=app, verbosity=2
            )
            if dev:
                try:
                    call_command(
                        "loaddata",
                        settings.THX_CORE_INIT_DATA_DEV_FILE_NAME,
                        app=app, verbosity=2
                    )
                except CommandError as e:
                    self.stdout.write(self.style.ERROR(f"{app}: {e}"))
